'----------------------------------------------------------
' This is simple visual RNG test, ported from a MASM source
' that I coded recently. It includes only a few generators
' because its purpose was just to try out FreeBASIC,
' and :) :) :)
'----------------------------------------------------------
option explicit

'$include: 'win\kernel32.bi'
'$include: 'win\user32.bi'
'$include: 'win\gdi32.bi'

' Globals.
dim shared selection as long
dim shared update_flag as long

' Declare WinMain as it would be for a C/C++ program.
' FB strings cannot be passed by value.
declare function WinMain  ( byval hInstance as long,_
                          byval hPrevInstance as long,_
                          lpCmdLine as string,_
                          byval nCmdShow as long)

' Specify WinMain as the function where execution will
' begin and pass it the normal parameter values (a null
' is passed for hPrevInstance because it is not used
' for Win32).
end WinMain( GetModuleHandle( null ), null, Command$, SW_NORMAL )

' Declare these functions here so they can be referenced
' before they are defined.
declare function fb_rand( byval range as long )
declare function crt_rand( byval range as long )
declare function bad_rand( byval range as long )
declare function pure_fb_rand( byval range as long )

' No declare is needed here because WndProc will not be
' referenced before it has been defined.
function WndProc (  byval hWin as long,_
                    byval uMsg as long,_
                    byval wParam as long,_
                    byval lParam as long )_
                    as long

    ' Handle only the necessary messages.
    select case uMsg
      case WM_COMMAND
        selection = wParam
        update_flag = true
      case WM_SIZE
        update_flag = true
      case WM_DESTROY
        PostQuitMessage 0
        exit function
    end select

    ' And pass the rest to the default window procedure.
    WndProc = DefWindowProc( hWin, uMsg, wParam, lParam )

end function

function WinMain( byval hInstance as long,_
                  byval hPrevInstance as long,_
                  lpCmdLine as string,_
                  byval nCmdShow as long)
    
    ' Local variables.
    dim hWnd as long
    dim hMenu as long
    dim hDC as long
    dim wwd as long
    dim wht as long
    dim wtx as long
    dim wty  as long
    dim xcoord as long
    dim ycoord as long
    dim counter as long
    dim start as long
    dim junk as long
    dim msg as MSG
    dim rect as RECT
    dim wcx as WNDCLASSEX
    dim wTitle as string
    dim szClassName as string
    dim szDisplayName as string

    ' Show our command line.
    'MessageBox null, lpCmdLine, "randtest", MB_OK

    ' Time the generators and show the results.
    start = GetTickCount
    for counter = 1 to 1000000
        junk = fb_rand( 100 )
    next
    MessageBox  null,_
                str$( GetTickCount - start ),_
                "fb_rand ticks",_
                MB_OK
    start = GetTickCount
    for counter = 1 to 1000000
        junk = crt_rand( 100 )
    next
    MessageBox  null,_
                str$( GetTickCount - start ),_
                "crt_rand ticks",_
                MB_OK
    start = GetTickCount
    for counter = 1 to 1000000
        junk = bad_rand( 100 )
    next
    MessageBox  null,_
                str$( GetTickCount - start ),_
                "bad_rand ticks",_
                MB_OK
    start = GetTickCount
    for counter = 1 to 1000000
        junk = pure_fb_rand( 100 )
    next
    MessageBox  null,_
                str$( GetTickCount - start ),_
                "pure_fb_rand ticks",_
                MB_OK

    ' Populate the WNDCLASSEX structure.
    szClassName = "randtest_class"
    szDisplayName = "RandTest"
    wcx.cbSize = len(WNDCLASSEX)
    wcx.style = CS_HREDRAW or CS_VREDRAW or CS_BYTEALIGNWINDOW
    wcx.lpfnWndProc = @WndProc()
    wcx.cbClsExtra = null
    wcx.cbWndExtra = null
    wcx.hInstance = hInstance
    wcx.hbrBackground = GetStockObject( WHITE_BRUSH )
    wcx.lpszMenuName = null
    wcx.lpszClassName = sadd( szClassName )
    wcx.hIcon = LoadIcon( null, IDI_APPLICATION )
    wcx.hCursor = LoadCursor( null, IDC_ARROW )
    wcx.hIconSm = LoadIcon( null, IDI_APPLICATION )

    ' And register it with Windows.
    RegisterClassEx( wcx )

    ' Specify the window size.
    wwd = 600
    wht = 400
 
     ' And calc the coordinates to center it on the screen.
    wtx = (GetSystemMetrics( SM_CXSCREEN ) / 2) - wwd / 2
    wty = (GetSystemMetrics( SM_CYSCREEN ) / 2) - wht / 2

    ' Create the window and save the handle.
    hwnd = CreateWindowEx(  WS_EX_OVERLAPPEDWINDOW,_
                            sadd( szClassName ),_
                            sadd( szDisplayName ),_
                            WS_OVERLAPPEDWINDOW,_
                            wtx,_
                            wty,_
                            wwd,_
                            wht,_
                            null,_
                            null,_
                            hInstance,_
                            null )

    ' Load the menu resource from the EXE file.
    hMenu = LoadMenu( hInstance, 600 )

    ' And assign it to the window.
    SetMenu( hWnd, hMenu )

    ' Set the window's show state.
    ShowWindow( hWnd, nCmdShow )

    ' And send the window its first WM_PAINT message.
    ' This is not actually necessary for this program,
    ' but it can be used where it is necessary to
    ' force an initial paint.
    UpdateWindow( hWnd )

    ' Get a device context for the client area of
    ' the window and save the returned handle.
    hDC = GetDC( hWnd )

    ' Enter the main message loop. We need to continuously
    ' update the client area of the Window, so we do it
    ' from the main message loop, and we need to retrieve
    ' the messages with PeekMessage instead of GetMessage
    ' because GetMessage would wait for a message to be
    ' posted to our thread's message queue before it
    ' returned. PM_REMOVE indicates that we want the
    ' messages removed from the message queue as we process
    ' them.
    do
      if( PeekMessage( msg, null, 0, 0, PM_REMOVE ) <> 0 ) then
        ' PeekMessage returned non-zero, indicating that a
        ' message is available. If the message was WM_QUIT,
        ' exit from the loop and return to Windows.
    
        if msg.message = WM_QUIT then exit do

        ' Otherwise translate any virtual key messages
        ' to character messages and dispatch the message
        ' to our window procedure.
        TranslateMessage( msg )
        DispatchMessage( msg )
      else
        ' PeekMessage returned zero, indicating that no
        ' mesages are available. Get the coordinates of
        ' our window's client area.
        GetClientRect( hWnd, rect )
        
        ' If the update flag is set erase the contents of
        ' the window and update the window title based on
        ' the current menu selection.
        if update_flag then
          update_flag = 0
          wTitle = ""
          FillRect( hDC, rect, wcx.hbrBackground )
          select case selection
            case 1000
              wTitle = "test: fb_rand "
              gosub updateTitle
            case 1001
              wTitle = "test: crt_rand "
              gosub updateTitle
            case 1002
              wTitle = "test: bad_rand "
              gosub updateTitle
            case 1003
              wTitle = "test: pure_fb_rand "
              gosub updateTitle
          end select
        end if
        
        ' Get random x and y pixel coordinates, based
        ' on the current menu selection.
        select case selection
          case 1000
            xcoord = fb_rand( rect.nRight )
            ycoord = fb_rand( rect.nBottom )
          case 1001
            xcoord = crt_rand( rect.nRight )
            ycoord = crt_rand( rect.nBottom )
          case 1002
            xcoord = bad_rand( rect.nRight )
            ycoord = bad_rand( rect.nBottom )
          case 1003
            xcoord = pure_fb_rand( rect.nRight )
            ycoord = pure_fb_rand( rect.nBottom )
        end select
      end if

      ' If a selection has been made, then draw a black
      ' pixel at the random coordinates. If no selection
      ' has been made then the loop runs so fast that at
      ' some locations the mouse cursor will not properly
      ' respond to mouse motions.
      if selection then SetPixel( hDC, xcoord, ycoord, 0 )

    loop

    ' Return the exit code in msg.wParam to the system.
    WinMain = msg.wParam
    
    ' Bypass the subroutine to avoid an exception.
    exit function

    ' Subroutines have no place in "structured" programming,
    ' but they sure are handy :)
  updateTitle:
    wTitle = wTitle + str$(rect.nRight)
    wTitle = wTitle + "x"
    wTitle = wTitle + str$(rect.nBottom)
    SetWindowText( hWnd, sadd( wTitle ) )
    return

end function

' ---------------------------------------------------------
' This function adapts the RND function to return a
' uniformly distributed random number in the interval
' 0 to range - 1.
' ---------------------------------------------------------
function fb_rand( byval range as long ) as long
    fb_rand = range * rnd()
end function

' ---------------------------------------------------------
' This function is an asm version of the single-threaded
' CRT rand function, based on the rand.c source that was
' distributed with the PSDK:
'
' return(((holdrand=holdrand*214013L+2531011L)>>16)&0x7fff);
'
' But modified to take a range argument and return a
' uniformly distributed random number in the interval
' 0 to range - 1, where range must be in the interval
' 1 to 32767.
' ---------------------------------------------------------
' Must initialize seed to non-zero value <= 7fffffffh.
dim shared crt_rand_seed as long = {12345678}
function crt_rand( byval range as long ) as long
    dim rval as long    

    asm
      mov   eax, [crt_rand_seed]
      mov   ecx, 214013
      mul   ecx
      add   eax, 2531011
      mov   [crt_rand_seed], eax
      shr   eax, 16
      and   eax, 32767
      xor   edx, edx      
      div   dword ptr[range]
      mov   [rval], edx
    end asm
    crt_rand = rval
end function

' ---------------------------------------------------------
' This is an example of a function that produces bad random
' numbers, included to demonstrate how the program output
' is sensitive to non-randomness in the generated numbers
' The generator itself is OK, but the code that scales the
' return value, while it is very fast, will work correctly
' only if range is a power of two. You can verify this by
' adjusting the client area of the window to 512x512.
' ---------------------------------------------------------
' Must initialize seed to non-zero value <= 7fffffffh.
dim shared bad_rand_seed as long = {12345678}
function bad_rand( byval range as long ) as long
    dim rval as long

    asm
      mov   eax, [bad_rand_seed]
      mov   ecx, 214013
      mul   ecx
      add   eax, 2531011
      mov   [bad_rand_seed], eax
      shr   eax, 16
      and   eax, 32767
      mov   edx, dword ptr[range]
      dec   edx           ' First part of EAX MOD range.
      and   eax, edx      ' Second part of EAX MOD range.
      mov   [rval], eax
    end asm
    bad_rand = rval
end function

' ---------------------------------------------------------
' This function is a pure FreeBASIC version of the single-
' threaded CRT rand function, based on the rand.c source
' that was distributed with the PSDK:
'
' return(((holdrand=holdrand*214013L+2531011L)>>16)&0x7fff);
'
' But modified to take a range argument and return a
' uniformly distributed random number in the interval
' 0 to range - 1, where range must be in the interval
' 1 to 32767.
' ---------------------------------------------------------
' Must initialize seed to non-zero value <= 7fffffffh.
dim shared pure_fb_rand_seed as long = {12345678}
function pure_fb_rand( byval range as long ) as long
    dim rval as unsigned long

    rval = pure_fb_rand_seed * 214013 + 2531011
    pure_fb_rand_seed = rval
    rval = rval SHR(16) and &h7fff

    pure_fb_rand = rval mod range

end function
